/**********************************************************************/
/*                                                                    */
/* Searches system drive(s) for .HLP and .INF files and creates       */
/* program objects for them in a destination folder.                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Release history:                                                   */
/*                                                                    */
/* 2001-02-22 : Bug fix: empty drive list failed to halt program.     */
/*              Bug fix: log file was not closed.                     */
/*              Changed log file handling.                            */
/*              Changed REXX utility functions loader.                */
/*              Added "found" counter (as opposed to "created").      */
/*              Cosmetic changes to code layout.                      */
/* 2000-05-23 : Bug fix: non-OS/2 HLP & INF files were being included.*/
/* 2000-03-21 : Log file function added.                              */
/* 1999-08-25 : Cosmetic changes to main routine and interface.       */
/* 1999-07-21 : Original program release.                             */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest version of this program can be found at                 */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 1999-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

signal on Halt  /* handle halt condition */

say 'This program searches one or more system drives for .HLP and .INF'  /* info */
say 'files and for each file found creates a VIEW.EXE program object'  /* info */
say 'in a destination folder.'  /* info */
say ''  /* empty line */

if (\LoadRexxUtils()) then  /* if we cannot use or load the Rexx utility functions */
do
 exit  /* just quit */
end

DriveList = GetDrives()  /* check the list of drives */

if (DriveList = '') then  /* if the list contained an error */
do
 exit  /* just quit */
end

Destination = GetDest()  /* get the destination folder */

if (Destination = '') then  /* if we have no destination */
do
 exit  /* just quit */
end

LogFile = Destination||'\InfoColl.log'  /* the default log file to use */

if (stream(LogFile,'C','OPEN WRITE') >< 'READY:') then  /* if we cannot open the log file for writing */
do
 say 'Cannot open log file : "'||LogFile||'"'  /* report */
 exit  /* just quit */
end

if (words(DriveList) > 1) then  /* if we have more than one drive */
do
 Plural = 's'  /* we need a plural "s" */
end
else  /* if we have only one drive to search */
do
 Plural = ''  /* we don't need the extra "s" */
end
say ''  /* empty line */
say 'Drive'||Plural||' to search:'||DriveList  /* report */
say 'Destination = '||Destination  /* report */
say 'Results will be logged in '||LogFile  /* report */
say ''  /* empty line */

TotalCount = 0  /* reset the counter */
do words(DriveList)  /* for each drive in the list */
 parse var DriveList Drive DriveList  /* get the next drive */
 TotalCount = TotalCount + CollectInfo(Drive,Destination,LogFile,'HLP')  /* look for HLP files*/
 TotalCount = TotalCount + CollectInfo(Drive,Destination,LogFile,'INF')  /* and look for INF files */
end

say ''  /* empty line */
say TotalCount||' program objects created in '||Destination  /* report */

call Halt  /* that's all, folks! */

/**********************************************************************/
GetDrives: procedure  /* gets a list of drives */
/**********************************************************************/

say 'Drive(s) to search (separate with spaces):'  /* info */
parse pull DrivesInput  /* get a list of drives to search */

DrivesInput = strip(DrivesInput,'B',' ')  /* get rid of any leading and trailing blanks */

if (DrivesInput = '') then  /* if we have no list of drives to check */
do
 say 'No drive list entered'  /* report */
 return ''  /* return empty */
end

DriveMap = sysdrivemap(,'USED')  /* get a list of all the drives currently in use */
DriveList = ''  /* start with nothing */

do words(DrivesInput)  /* for each of the drives requested */
 parse var DrivesInput OneDrive DrivesInput  /* get the drive name */
 OneDrive = translate(strip(OneDrive,'T',':'))||':'  /* reshape it into an uppercase word followed by one colon */
 if (pos(OneDrive,DriveMap) = 0) then  /* if it is not in the official list */
 do
  call beep 333,333  /* signal */
  say OneDrive||' is not a valid drive'  /* report */
  DriveList = ''  /* make sure we return no result */
 end
 else  /* if it is in the list */
 do
  DriveList = DriveList||' '||OneDrive  /* add it to our own list */
 end
end

return DriveList  /* end of CheckDrives */

/**********************************************************************/
GetDest: procedure  /* gets destination folder */
/**********************************************************************/

say 'Enter the name of the destination folder:'  /* put out a prompt */
parse pull Location  /* get the location */

Location = strip(Location,'B',' ')  /* get rid of any leading and trailing blanks */
Location = strip(Location,'T','\')  /* get rid of any trailing backslash */

if (Location = '') then  /* if we have nothing */
do
 say 'No location entered'  /* report */
 return ''  /* return empty */
end

DriveLetter = filespec('D',Location)  /* get any drive letter from the new location */
if (DriveLetter = '') then  /* if there is no drive letter */
do
 say 'The location must include a drive letter'  /* report */
 return ''  /* return empty */
end

call sysfiletree Location,'Loc.','DO'  /* look for the new location */
if (Loc.0 = 0) then  /* if it is not found */
do
 call charout ,'"'||Location||'" does not exist; should the folder be created? (Y/N) Y'||d2c(8)  /* put out a query text */
 Reply = sysgetkey('ECHO')  /* get a character from STDIN */
 say ''  /* skip to the next line */
 if (pos(Reply,d2c(13)||'Yy+') = 0) then  /* if we do not get the O.K. */
 do
  Location = ''  /* return empty */
 end
 else  /* if we get the O.K. to create the folder */
 do
  say 'Creating folder: '||Location  /* report */
  if (sysmkdir(Location) >< 0) then  /* if we cannot create the new location folder */
  do
   say 'Cannot create '||Location  /* report */
   Location = ''  /* return empty */
  end
 end
end

return Location  /* return with the new location */

/**********************************************************************/
CollectInfo: procedure  /* does the actual collecting */
/**********************************************************************/

parse arg Drive,Destination,LogFile,FileType  /* get the arguments */

say 'Looking for '||FileType||' files on '||Drive  /* report */

FoundCounter = 0  /* start with 0 files found */
CreateCounter = 0  /* start with 0 files created */

call sysfiletree Drive||'\*.'||FileType,'Files.','FOS'  /* look for the files in the root and subdirectories */

do Index = 1 to Files.0  /* for each of the files in filespec, do */

 FileTitle = '_'  /* dummy object title */

 call stream Files.Index,'C','OPEN READ'  /* open the source file for reading */
 Input = CharIn(Files.Index,1,3)  /* get the first three bytes */

 If (Input = 'HSP') then  /* if it is a HLP or INF file */
 do
  FileTitle = ''  /* clear object title */
  Position = 108  /* this is where the file title should be in the file */
  Input = CharIn(Files.Index,Position,1)  /* read the byte at offset 108 in the file */
  do while (C2D(Input) >< 0)  /* as long as we don't have a null byte */
   FileTitle = FileTitle||Input  /* add the byte to the object title string */
   Position = Position + 1  /* move one byte up */
   Input = CharIn(Files.Index,Position,1)  /* and read that byte */
  end
 end

 call stream Files.Index,'C','CLOSE'  /* close the source file */

 if (FileTitle >< '_') then  /* If it is a file we want */
 do

  if (FileTitle = '') then  /* If no title was found */
  do
   FileTitle = '_Untitled'  /* create a dummy title */
  end

  ObjectTitle = FileTitle||d2c(10)||'('||Files.Index||')'  /* add the file path on a separate line to complete the title */
  Params = 'EXENAME=VIEW.EXE;PARAMETERS='||Files.Index||';'  /* the parameters to use for the program object */

  call lineout LogFile,'File  : '||Files.Index  /* log a line */

  FoundCounter = FoundCounter + 1  /* up the files found counter */

  if (syscreateobject('WPProgram',ObjectTitle,Destination,Params)) then  /* if we can create a program object */
  do
   CreateCounter = CreateCounter + 1  /* up the counter */
   call lineout LogFile,'Title : '||FileTitle  /* log a line */
  end
  else  /* if we cannot create it */
  do
   say 'Cannot create program object for : '||Files.Index  /* report */
   call lineout LogFile,'Could not create program object'  /* log a line */
  end

  call lineout LogFile,''  /* log an empty line */

 end

end

say FoundCounter||' '||FileType||' files found'  /* report */
say CreateCounter||' '||FileType||' objects created'  /* report */

return CreateCounter  /* end of CollectInfo */

/**********************************************************************/
LoadRexxUtils: procedure  /* loads REXX utility functions if necessary */
/**********************************************************************/

Success = 1  /* all is well to start with */

if (rxfuncquery('SysLoadFuncs') >< 0) then  /* if we have to load the REXX utility functions */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  call sysloadfuncs  /* call the general loading function */
  say 'Rexx utility functions loaded'  /* report */
 end
 else  /* if we cannot register the general loading function */
 do
  say  'Cannot load Rexx utility functions'  /* report */
  Success = 0  /* no success */
 end
end

return Success  /* end of LoadRexxUtils */

/**********************************************************************/
Halt:  /* handles halt condition */
/**********************************************************************/

call stream LogFile,'C','CLOSE'  /* close the log file */

exit  /* just quit */
